@@ -28,6 +28,7 @@ __copyright__ = "Copyright (c) 2009 Sun Microsystems Inc." \
 __license__   = "LGPL"
 
 import pyatspi
+import re
 import sys
 import time
 import traceback
@@ -238,11 +239,14 @@ class Generator:
             else:
                 firstTimeCalled = False
 
-            msg = '%s GENERATOR: Starting generation for %s' % (self._mode.upper(), obj)
+            msg = '%s GENERATOR: Starting %s generation for %s (%s)' % \
+                (self._mode.upper(), args.get('formatType'), obj, args.get('role'))
             debug.println(debug.LEVEL_INFO, msg, True)
 
-            # Reset 'usedDescriptionForName' if a previous generator used it.
+            # Reset 'usedDescriptionFor*' if a previous generator used it.
             self._script.pointOfReference['usedDescriptionForName'] = False
+            self._script.pointOfReference['usedDescriptionForUnrelatedLabels'] = False
+            self._script.pointOfReference['usedDescriptionForAlert'] = False
 
             debuginfo = lambda x: self._resultElementToString(x, False)
             assert(formatting)
@@ -396,12 +400,26 @@ class Generator:
             descendant = self._script.utilities.realActiveDescendant(obj)
             name = self._generateName(descendant)
 
+        # If we don't have a label, always use the name.
+        if not label:
+            return name
+
         result.extend(label)
-        if not len(label):
-            result.extend(name)
-        elif len(name) and name[0].split() != label[0].split() \
-             and not label[0].startswith(name[0]):
-            result.extend(name)
+        if not name:
+            return result
+
+        # Try to eliminate names which are redundant to the label.
+        # Convert all non-alphanumeric characters to space and get the words.
+        nameWords = re.sub(r"[\W_]", " ", name[0]).split()
+        labelWords = re.sub(r"[\W_]", " ", label[0]).split()
+
+        # If all of the words in the name are in the label, the name is redundant.
+        if set(nameWords).issubset(set(labelWords)):
+            msg = "GENERATOR: name '%s' is redundant to label '%s'" % (name[0], label[0])
+            debug.println(debug.LEVEL_INFO, msg, True)
+            return result
+
+        result.extend(name)
         return result
 
     def _generateLabelOrName(self, obj, **args):
@@ -415,6 +433,18 @@ class Generator:
 
         return result
 
+    def _generateUnrelatedLabelsOrDescription(self, obj, **args):
+        result = self._generateUnrelatedLabels(obj, **args)
+        if result:
+            self._script.pointOfReference['usedDescriptionForUnrelatedLabels'] = False
+            return result
+
+        result = self._generateDescription(obj, **args)
+        if result:
+            self._script.pointOfReference['usedDescriptionForUnrelatedLabels'] = True
+
+        return result
+
     def _generateDescription(self, obj, **args):
         """Returns an array of strings fo use by speech and braille that
         represent the description of the object, if that description
@@ -424,6 +454,12 @@ class Generator:
         if self._script.pointOfReference.get('usedDescriptionForName'):
             return []
 
+        if self._script.pointOfReference.get('usedDescriptionForAlert'):
+            return []
+
+        if self._script.pointOfReference.get('usedDescriptionForUnrelatedLabels'):
+            return []
+
         role = args.get('role', obj.getRole())
 
         # Unity Panel Service menubar items are labels which claim focus and
@@ -446,9 +482,24 @@ class Generator:
 
         result = []
         if description:
+            try:
+                tokens = self._script.formatting[self._mode][role][args.get('formatType')].split()
+                isLabelAndName = 'labelAndName' in tokens
+                isLabelOrName = 'labelOrName' in tokens
+            except:
+                isLabelAndName = False
+                isLabelOrName = False
+
             label = self._script.utilities.displayedLabel(obj) or ""
             desc = description.lower()
-            if not (desc in name.lower() or desc in label.lower()):
+            canUse = True
+            if isLabelAndName:
+                canUse = not desc in name.lower() and not desc in label.lower()
+            elif isLabelOrName and label:
+                canUse = not desc in label.lower()
+            elif isLabelOrName and name:
+                canUse = not desc in name.lower()
+            if canUse:
                 result.append(obj.description)
 
         if not result:
@@ -475,6 +526,9 @@ class Generator:
 
         return self._generateStatusBar(obj, **args)
 
+    def _generateTermValueCount(self, obj, **args):
+        return []
+
     #####################################################################
     #                                                                   #
     # Image information                                                 #
@@ -930,7 +984,8 @@ class Generator:
         rows, cols = self._script.utilities.rowAndColumnCount(obj)
 
         # This suggests broken or missing table interface.
-        if rows < 0 or cols < 0:
+        if (rows < 0 or cols < 0) \
+           and not self._script.utilities.rowOrColumnCountUnknown(obj):
             return []
 
         # This can happen if an author uses ARIA incorrectly, e.g. a grid whose
@@ -1168,7 +1223,10 @@ class Generator:
             return self._generateDisplayedText(rad, **args)
 
         content = set([self._script.utilities.displayedText(x).strip() for x in rad])
-        return [" ".join(filter(lambda x: x, content))]
+        rv = " ".join(filter(lambda x: x, content))
+        if not rv:
+            return self._generateDisplayedText(rad, **args)
+        return [rv]
 
     def _generateRealActiveDescendantRoleName(self, obj, **args ):
         """Objects, such as tables and trees, can represent individual cells
@@ -1297,6 +1355,16 @@ class Generator:
             return 'ROLE_CONTENT_MARK'
         if self._script.utilities.isContentSuggestion(obj):
             return 'ROLE_CONTENT_SUGGESTION'
+        if self._script.utilities.isDescriptionList(obj):
+            return pyatspi.ROLE_DESCRIPTION_LIST
+        if self._script.utilities.isDescriptionListTerm(obj):
+            return pyatspi.ROLE_DESCRIPTION_TERM
+        if self._script.utilities.isDescriptionListDescription(obj):
+            return pyatspi.ROLE_DESCRIPTION_VALUE
+        if self._script.utilities.isFeedArticle(obj):
+            return 'ROLE_ARTICLE_IN_FEED'
+        if self._script.utilities.isFeed(obj):
+            return 'ROLE_FEED'
         if self._script.utilities.isLandmark(obj):
             if self._script.utilities.isLandmarkRegion(obj):
                 return 'ROLE_REGION'
